* Generic session options - page and line size, no page breaks;
OPTIONS nonumber nodate nocenter pagesize=MAX linesize=100 formdlim='-';											
* Kill default titles on output; 
TITLE; ODS LISTING;

* Location for original SAS files for these models;
%LET example =  C:\Dropbox\_Archive\Example Data\School Data;
LIBNAME example "&example.";
	
* Bring in SAS file with macro programs;
%INCLUDE "&example.\MLM_Macros.sas";

* Save SAS syntax and output to permanent text file;
PROC PRINTTO PRINT="&example.\CLDP945_Example06a_SAS_Output.txt" 
			LOG="&example.\CLDP945_Example06a_SAS_Output.txt"; RUN;

* Importing data into work library;
DATA work.grade10; SET example.grade10school; 
LABEL	studentID="studentID: Student ID number"
		schoolID=	"schoolID: School ID number"
		frlunch=	"frlunch: 0=No, 1=Free/Reduced Lunch"
		math=	"math: Math Test Score Outcome";
	* Selecting cases that are complete for analysis variables;
	IF NMISS(studentID, schoolID, frlunch, math)>0 THEN DELETE;
RUN;

* Getting school means;
PROC SORT DATA=work.grade10; BY schoolID studentID; RUN;
PROC MEANS NOPRINT N DATA=work.grade10; 
	 BY schoolID;
	 VAR frlunch math;
	 OUTPUT OUT=work.SchoolMeans 
		    MEAN(frlunch math)= SMfrlunch SMmath; 
RUN;

* Labeling new school mean variables;
DATA work.SchoolMeans; SET work.SchoolMeans;
	SchoolN = _FREQ_;	* Saving N per school;
	DROP _TYPE_ _FREQ_;	* Dropping unneeded SAS-created variables;
LABEL	SMfrlunch=	"SMfrlunch: School Mean 0=No, 1=Free/Reduced Lunch"
		SMmath=		"SMmath: School Mean Math Outcome"
		SchoolN=		"SchoolN: # Students Contributing Data";
RUN;

* Merging school means back with individual data;
DATA work.grade10; MERGE work.grade10 work.SchoolMeans; BY schoolID;
	* Selecting only schools with data from at least 30 students;
	IF SchoolN < 31 THEN DELETE;
RUN;

* Outputting table of descriptives;
TITLE "Getting means to center predictors with";
PROC MEANS N MEAN STDDEV MIN MAX DATA=work.grade10; 
	 VAR math frlunch SMmath SMfrlunch SchoolN; 
RUN; TITLE;

* Centering school mean predictor;
DATA grade10; SET grade10;
	SMfrlunch30 = SMfrlunch - .30; LABEL SMfrlunch30= "SMfrlunch30: 0=.30)";
RUN;

******************************************************************************;
****** 					BEGIN Example 6a MODELS				  ******;
******************************************************************************;

TITLE1 "SAS Model 1: 2-Level Empty Means, Random Intercept for Math Outcome";
PROC MIXED DATA=work.grade10 NOCLPRINT COVTEST NAMELEN=100 IC METHOD=ML;
	CLASS schoolID studentID;
	MODEL math = / SOLUTION DDFM=Satterthwaite;
	RANDOM INTERCEPT / TYPE=UN SUBJECT=schoolID; 
	ODS OUTPUT CovParms=CovEmpty InfoCrit=FitEmpty; RUN;

TITLE1 "SAS Model 2: Adding Fixed Effect of Student Free/Reduced Lunch";
PROC MIXED DATA=work.grade10 NOCLPRINT COVTEST NAMELEN=100 IC METHOD=ML;
	CLASS schoolID studentID;
	MODEL math = frlunch / SOLUTION DDFM=Satterthwaite;
	RANDOM INTERCEPT / TYPE=UN SUBJECT=schoolID; 
	ODS OUTPUT CovParms=CovFR1 InfoCrit=FitFR1; RUN; 

* Calculate PseudoR2 relative to empty model;
%PseudoR2(NCov=2, CovFewer=CovEmpty, CovMore=CovFR1);

TITLE1 "SAS Model 3: Adding Fixed Effect of School Proportion Free/Reduced Lunch";
PROC MIXED DATA=work.grade10 NOCLPRINT COVTEST NAMELEN=100 IC METHOD=ML;
	CLASS schoolID studentID;
	MODEL math = frlunch SMfrlunch30 / SOLUTION DDFM=Satterthwaite OUTPM=work.LunchSave;
	RANDOM INTERCEPT / TYPE=UN SUBJECT=schoolID; 
	ODS OUTPUT CovParms=CovFR2 InfoCrit=FitFR2;
    ESTIMATE "FR Lunch Between-School Effect"  frlunch 1 SMfrlunch30 1; RUN; 
PROC CORR NOSIMPLE DATA=work.LunchSave; VAR math pred; RUN;

* Calculate PseudoR2 relative to previous model;
%PseudoR2(NCov=2, CovFewer=CovFR1, CovMore=CovFR2);

* Calculate PseudoR2 relative to empty model (total for FRlunch);
%PseudoR2(NCov=2, CovFewer=CovEmpty, CovMore=CovFR1);

TITLE1 "SAS Model 4: Adding Random Effect of Student Free/Reduced Lunch";
PROC MIXED DATA=work.grade10 NOCLPRINT COVTEST NAMELEN=100 IC METHOD=ML;
	CLASS schoolID studentID;
	MODEL math = frlunch SMfrlunch30 / SOLUTION DDFM=Satterthwaite;
	RANDOM INTERCEPT frlunch / G TYPE=UN SUBJECT=schoolID; 
	ODS OUTPUT CovParms=CovFR2RandFR1 InfoCrit=FitFR2RandFR1; RUN;

* Calculate difference in model fit relative to fixed-FRlunch-only model;
%FitTest(FitFewer=FitFR2, FitMore=FitFR2RandFR1);

TITLE1 "SAS Model 5: Adding Cross-Level Interaction of Student by School Free/Reduced Lunch";
PROC MIXED DATA=work.grade10 NOCLPRINT COVTEST NAMELEN=100 IC METHOD=ML;
	CLASS schoolID studentID;
	MODEL math = frlunch SMfrlunch30 frlunch*SMfrlunch30 
		/ SOLUTION DDFM=Satterthwaite;
	RANDOM INTERCEPT frlunch / G TYPE=UN SUBJECT=schoolID; 
	ODS OUTPUT CovParms=CovInt1 InfoCrit=FitInt1; RUN;

* Calculate PseudoR2 for both FRlunch effects relative to random FRlunch;
%PseudoR2(NCov=4, CovFewer=CovFR2RandFR1, CovMore=CovInt1);

TITLE1 "SAS Model 6: Adding Level-2 Interaction of Quadratic School Free/Reduced Lunch";
PROC MIXED DATA=work.grade10 NOCLPRINT COVTEST NAMELEN=100 IC METHOD=ML;
	CLASS schoolID studentID;
	MODEL math = frlunch SMfrlunch30 frlunch*SMfrlunch30 SMfrlunch30*SMfrlunch30
		/ SOLUTION DDFM=Satterthwaite OUTPM=work.TotalSave;
	RANDOM INTERCEPT frlunch / G TYPE=UN SUBJECT=schoolID; 
	ODS OUTPUT CovParms=CovInt2 InfoCrit=FitInt2;
    ESTIMATE "FR Lunch Between-School Main Effect"  frlunch 1 SMfrlunch30 1;
	ESTIMATE "FR Lunch Between-School Interaction"  frlunch*SMfrlunch30 1 SMfrlunch30*SMfrlunch30 1;
RUN; PROC CORR NOSIMPLE DATA=work.TotalSave; VAR math pred; RUN;

* Calculate PseudoR2 relative to level-1 lunch interaction only model;
%PseudoR2(NCov=4, CovFewer=CovInt1, NameFewer=LunchInt1, CovMore=CovInt2, NameMore=LunchInt2);

* Close output;
PROC PRINTTO; RUN;
